import DebounceThrottle from './DebounceThrottle'
import io from 'socket.io-client'
import {stringify} from 'flatted/esm'

export const launchReactComponentTreeViewer = (port) => {
  if (window.__REACT_DEVTOOLS_GLOBAL_HOOK__) {
    console.log('react-component-tree-viewer launched')
    const socket = io(`http://localhost:${port.toString()}`)
    const debounceThrottle = new DebounceThrottle()

    const hook = window.__REACT_DEVTOOLS_GLOBAL_HOOK__
    hook.onCommitFiberRoot = ((originalHook) => (...args) => {
      const [, root] = args

      debounceThrottle.throttle(() => {
        const data = root.current.stateNode.current.child
        const stringifyReplacer = (_, value) => typeof value === 'function' ? value.name : value

        socket.emit('sendTree', stringify(data, stringifyReplacer))
      }, 1000)

      return originalHook(...args)
    })(hook.onCommitFiberRoot)
  }
}

